#include "txtbl.h"

#define BUF_SZ		4096

static int bak = 0;

static int
getch(void)
{
	int ret;

	if(bak){
		ret = bak;
		bak = 0;
		return ret;
	}
	return getchar();
}

static void
bakch(int ch)
{
	bak = ch;
}

static int
is_skip_spc(int ch, int spc_flag)
{
	return spc_flag ? ch == '\n' : isspace(ch);
}

static void
cell_set_str(struct cell *p, char *s)
{
	int i, w;

	for(; is_skip_spc(*s, p->spc_top); s++);
	for(i=strlen(s)-1; is_skip_spc(s[i], p->spc_tail); i--) s[i] = '\0';

	if((p->s = strdup(s)) == NULL){
		fprintf(stderr, "No mem\n");
		exit(1);
	}

	p->sh = 1;
	p->sw = 0;
	w = 0;
	for(s=p->s; *s; s++){
		if(*s == '\n'){
			p->sh++;
			w = 0;
			continue;
		}
		w++;
		if(w > p->sw) p->sw = w;
	}
}

static int
eof_chk(char *buf, int n)
{
	int i;

	for(i=0; i<n; i++) if(!isspace(buf[i])) return 0;
	return 1;
}

int
cell_get(struct cell *p)
{
	int ch, i, esc, n;
	char buf[ BUF_SZ ];

	p->w = p->h = 1;
	p->s = NULL;
	p->div = '-';
	p->cen = 0;
	p->spc_top = p->spc_tail = 0;

	esc = 0;
	i = 0;
	while((ch = getch()) != EOF){
		if(i >= BUF_SZ - 1){
			fprintf(stderr, "Give up , buf idx=%d\n", i);
			exit(1);
		}
		if(esc == 0){
			if(ch == '\\') esc = 1;
			else buf[i++] = ch;
			continue;
		}
		if(esc == 'w' || esc == 'h'){
			if(isdigit(ch)){
				n *= 10;
				n += ch - '0';
			}else{
				bakch(ch);
				if(n < 1) n = 1;
				if(esc == 'w') p->w = n;
				else p->h = n;
				esc = 0;
			}
			continue;
		}
		switch(ch){
		case '|':
			p->div = '|';
		case '-':
			break;
		case 'c':
			p->cen = 1;
			esc = 0;
			break;
		case 'w':
			n = 0;
			esc = 'w';
			break;
		case 'h':
			n = 0;
			esc = 'h';
			break;
		case '~':
			p->spc_top = 1;
			esc = 0;
			break;
		case '$':
			p->spc_tail = 1;
			esc = 0;
			break;
		case 'n':
			buf[i++] = '\n';
			esc = 0;
			break;
		case 't':
			buf[i++] = '\t';
			esc = 0;
			break;
		case '\n':
			esc = 0;
			break;
		case '\\':
		default:
			buf[i++] = '\\';
			esc = 0;
			break;
		}
		if(ch == '|' || ch == '-') break;
	}
	if(ch == EOF && eof_chk(buf, i)) return EOF;

	buf[i] = '\0';
	cell_set_str(p, buf);

	return 0;	
}

/* EOF */
